#ifndef CMBCOSMO__H
#define CMBCOSMO__H

#include "iostream"
#include "cosmology.h"

class CmbCosmo : public Cosmology
{
  double obh2;
  double h0;
  
  Cosmology *actualCosmo;


public:
  CmbCosmo(const Cosmology &C, const double OmegabH2, const double H0);

  double H0() const { return h0;}
  double OBH2() const { return obh2;}
  
  double ns; // for some CMB  prior
  double delta; // normalisation of P(k), for Huterer Planck  prior
  
  //Extract the underlying cosmology
  Cosmology* ExtractCosmo() const { return actualCosmo;}
  
  //! pipe the virtual routines through:
  double Omegam() const { return actualCosmo->Omegam();}
  double E(const double z) const { return actualCosmo->E(z);}

  int NPar() const { return actualCosmo->NPar();}

  double W(const double &Z) const { return actualCosmo->W(Z);}

  double integ_hinv(const double &ZMin, const double &ZMax, 
		    const int NStep=100) const 
  { return actualCosmo->integ_hinv(ZMin, ZMax, NStep);}

  void dump(std::ostream &stream) const;

  Cosmology* Clone() const {return new CmbCosmo(*actualCosmo,obh2,h0);}

  void SetParam(const int IPar, const double Val);

  double GetParam(const int IPar) const {return actualCosmo->GetParam(IPar);}
 
  /* zstar */
  double zstar() const;

  /* zdrag */
  double zdrag() const;

  /* comoving sound horizon */
  double rs(const double Z) const;


  double l_a(const double Z, const int NStep=100) const;

  double R(const double Z, const int NStep=100) const;

  //  int GetParamRank(const char *Name) const;


  ~CmbCosmo() { delete actualCosmo;}

 private :
  CmbCosmo(CmbCosmo &); // copies not handled
  void operator =(const CmbCosmo &);

};


#endif

