#include <string>
#include <vector>
#include <fstream>
#include <map>

#include "measureddata.h"
#include "param.h"
#include "snfitexception.h"

using namespace std;

#define _GNU_SOURCE 1
#ifndef __USE_GNU
#define __USE_GNU
#endif
#include <fenv.h>


static void usage() {
  cerr << "convert_lightcurves lightcurve1 lightcurve2 -o newlightcurvesfile (-p salt2_results.dat) (-k key val ...) (-r key ...)" << endl;
  exit(EXIT_FAILURE); 
}

int main(int nargs, char **args)
{
  
  // defaults
  // ------------------------------------------------------
  vector<string> lc;
  string output_filename = "";
  string parameter_filename = "";
  map<string,string> globalkeys;
  vector<string> keys2remove;
  
  // read arguments
  // ------------------------------------------------------
  if (nargs <2) usage();
  for (int i=1; i<nargs; ++i)
    {
      char *arg = args[i];
      if (arg[0] != '-')
	{
	  lc.push_back(arg);continue;
	}
      switch (arg[1])
	{
	case 'o' : output_filename = args[++i]; break;
	case 'p' : parameter_filename = args[++i]; break;
	case 'k': {string key=args[++i]; string val=args[++i]; globalkeys[key]=val;} break;
	case 'r': {keys2remove.push_back(args[++i]);} break;
	default : usage();
	}
    }

  if(lc.size()==0)
    usage();
  if(output_filename=="")
    usage();
  
  try {
    
    FitParams* params=NULL;
    if(parameter_filename!="") {
      params=new FitParams();
      params->read(parameter_filename);
    }
    MeasuredData data(lc,true);

    for(map<string,string>::const_iterator it=globalkeys.begin(); it != globalkeys.end(); ++it)
      data.infos.AddKey(it->first,it->second);
    for(size_t k=0;k<keys2remove.size();k++)
      data.infos.Erase(keys2remove[k]);

    data.WriteLightCurvesFile(output_filename,params);
  }   catch (SnfitException e)
    {
      e.PrintMessage(cout);
      return EXIT_FAILURE;
    }
  return EXIT_SUCCESS;
}

