#include <string>
#include <vector>
#include <fstream>
#include <map>

#include "measureddata.h"
#include "param.h"
#include "snfitexception.h"

using namespace std;

#define _GNU_SOURCE 1
#ifndef __USE_GNU
#define __USE_GNU
#endif
#include <fenv.h>


static void usage() {
  cerr << "convert_spectrum input output" << endl;
  cerr << "-k key val : add global key to output file" << endl;
  cerr << "-c lambda_min lambda_max : mask observation outside lambda range (obs. frame)" << endl;
  cerr << "-r key (remove this key)" << endl;
  exit(EXIT_FAILURE); 
}

int main(int nargs, char **args)
{
  
  
  // read arguments
  // ------------------------------------------------------
  if (nargs <3) usage();
  string input_filename = args[1];
  string output_filename = args[2];
  map<string,string> globalkeys;
  double lambda_min = -1;
  double lambda_max = -1;
  vector<string> keys2remove;
  
  for (int i=3; i<nargs; ++i)
    {
      char *arg = args[i];
      if (arg[0] != '-')
	{
	  usage();
	}
      switch (arg[1])
	{
	case 'k': {string key=args[++i]; string val=args[++i]; globalkeys[key]=val;} break;
	case 'c': lambda_min = atof(args[++i]); lambda_max = atof(args[++i]);  break;
	case 'r': {keys2remove.push_back(args[++i]);} break;
	    
	default : usage();
	}
    }
  
  MeasuredSpectrum spec(input_filename,false,false);

  // un-validate ground-based UV obs
  if(lambda_max>0)
    for(MeasuredSpectrum::iterator it=spec.begin();it!=spec.end();it++) {
      if((*it)->Wavelength()<lambda_min || (*it)->Wavelength()>lambda_max)
	(*it)->SetValid(0);
    }
  for(map<string,string>::const_iterator it=globalkeys.begin(); it != globalkeys.end(); ++it)
    spec.infos.AddKey(it->first,it->second);
  for(size_t k=0;k<keys2remove.size();k++)
    spec.infos.Erase(keys2remove[k]);
  spec.Write(output_filename);
  
  return EXIT_SUCCESS;
}

