// This may look like C code, but it is really -*- C++ -*-
//
// $Id: datacards.h,v 1.5 2014/04/29 13:29:33 nrl Exp $
//
// Datacards, acquisition EROS II
//
//
// Eric Aubourg, Decembre 95
// Reza Ansari, Aout 96
//
// DAPNIA/SPP (Saclay) / CEA    LAL - IN2P3/CNRS  (Orsay)

#ifndef DATACARDS_SEEN
#define DATACARDS_SEEN
// This may look like C code, but it is really -*- C++ -*-
//
// $Id: datacards.h,v 1.5 2014/04/29 13:29:33 nrl Exp $
//




//#include "defs.h"
#include <string>
#include <cstdlib>
#include <cstdio>


struct MissingKeyException : public std::string
{
  MissingKeyException(const std::string &Message) : std::string(Message) {}
};





#include <functional>
#include <list>
#include <vector>

typedef int (*ProcCard)(std::string const& key, std::string const& toks);
 
class DataCards  {
public:
  DataCards();                  
  DataCards(std::string const& fn);

  // nom dans variable d'environnement PEIDA_DATACARDS
  // par defaut, peida.datacards                        
  // Si pas chemin complet, on tente dans repertoire 
  // en cours, puis dans PEIDA_WORK

  virtual ~DataCards() {}

  void    AddProcF(ProcCard f, std::string const& mtch="*");

  void    Clear();
  void    ReadFile(std::string const& fn);
  void    AppendCard(std::string const& line);

  int     NbCards() const;
  bool    HasKey(std::string const& key) const;
  int     NbParam(std::string const& key) const;
  std::string  SParam(std::string const& key, int numero = 0, std::string def="") const;
  long    IParam(std::string const& key, int numero = 0, long def = 0) const;
  double  DParam(std::string const& key, int numero = 0, double def = 0) const;
  std::vector<std::string> AllParams(const std::string &Key) const; 
  friend std::ostream& operator << (std::ostream& s, const DataCards &c);

public:
  struct Card {
    std::string kw;
    std::vector<std::string> tokens;
    //    STRUCTCOMPF(Card,kw)
      };
  typedef std::list<Card> CardList;
  struct CrdPF {
    ProcCard pf;
    std::string  patt;
    // STRUCTCOMPF(CrdPF,pf)
      };
  typedef std::list<CrdPF> CrdPFList;
protected:
  CardList cards;
  CrdPFList cpfs;
  std::string fileName;

  void  DoReadFile(std::string const& fn);

  int   ApplyPF(CrdPF & cpf, std::string const& key, std::string const& toks);
  int   ApplyPFL(std::string const& key, std::string const& toks);

  void  RemoveCard(std::string const& key);
   
  const Card* FindKey(std::string const& key, const bool ThrowIfAbsent = true) const;
  struct KeyEq : std::binary_function<Card, std::string, bool> {
    bool operator()(const Card& x, const std::string& y) const { return x.kw == y; }
  };
};


#endif
