#ifndef FILTER__H
#define FILTER__H


#include "sampledfunction.h"
#include "globalval.h"


//! just a wrapper for a Filter defined through a table of transmissions. Wavelengthes expected in Angstroms.
class Filter : public General1DFunction {
  
 private:
  std::string filterName, instrumentName;
  GlobalVal glob;
  std::string tag;
  
 public : 
  //! - 
  Filter(const std::string &FileName, const std::string &FilterName, 
	 const std::string &InstrumentName, const std::string &Tag="") : 
    General1DFunction(FileName), 
    filterName(FilterName), 
    instrumentName(InstrumentName), 
    glob(FileName), tag(Tag)
      {};

 
    //!-
  Filter(const General1DFunction &func,const std::string &FilterName, 
	 const std::string &InstrumentName)
    : General1DFunction(func), 
      filterName(FilterName), 
    instrumentName(InstrumentName)
	{};
  //! - 
      /*
	Filter(const std::string &FilterName, const std::string &InstrumentName) :
	filterName(FilterName), instrumentName(InstrumentName) {};
      */
  //! default constructor
  Filter() : filterName("NoName"), instrumentName("?") {};

  //! names that combine instrument name and filter name
  /*! do not change this routine without updating accordingly 
    InstrumentNameFromFilterName */
  std::string Name() const { return instrumentName+"::"+filterName;};

  //! -
  const std::string& InstrumentName() const { return instrumentName;}

  //! -
  const std::string& Band() const {return filterName;};

  //! -
  const std::string& Tag() const {if (tag != "") return tag; else return Band();};


  //! -
  const GlobalVal &Globals() const { return glob;}
  
  //! - 
  friend std::ostream& operator << (std::ostream &stream, const Filter& Filter);
  
  //! just the mean 
  double CentralWaveLength() const;

  //! 
  bool Write(const std::string &FileName) const;


};

//! extract the instrument name from Filter::Name() output
/*! use Filter::InstrumentName() if you hold an actual filter, rather than just its name */
std::string InstrumentNameFromFilterName(const std::string &FilterName);

#endif /* FILTER__H */
