#ifndef GALAXY_TEMPLATE_H
#define GALAXY_TEMPLATE_H

#include <string>
#include <map>
#include "sampledfunction.h"
#include "matvect.h"

// when sfr = 0
#define SSFR_MIN_VALUE -100
// what does zero means :
#define U_SSFR_MIN_VALUE 1.e-20
// when no value is available
#define SSFR_BAD_VALUE -1000



class GalaxyTemplate {
  
  
  
 public :
  
    //x = age for a Pegase template, x=tau or mean star pop age  for the exponential ... templates. Here x is called "the type index".
    //y=lambda
  Grid2DFunction continuum; 
  const double *GalaxyTemplate_Indexation_vals() const { return continuum.xvals();}
  const int GalaxyTemplate_Indexation_N() const { return continuum.NstepX();}
  
  bool HasLineSpectrum ;
  Grid2DFunction lines;
  Vect lines_fvals_buffer;
  void LoadTemplates(const std::string& cardfilename="");
  
  // normalised star formation rate in Mdot.Myr^-1 (for Mtot = 1 solar mass)
  General1DFunction SFR ;
  // normalised mass in stars (for Mtot = 1 solar mass)
  General1DFunction Stellar_Mass ;
  // normalised mass in white dwarfs (for Mtot = 1 solar mass)
  General1DFunction Stellar_WD_Mass ;
  //  mean metallicity metallicity of stars averaged on luminosity
  General1DFunction Star_Lum_Metallicity ;
  //  metallicity of gas 
  General1DFunction Gas_Metallicity ;
  // mean stellar age averaged on mass
  General1DFunction Stellar_Age_Mass ;
  // mean stellar age averaged on bolometric lum.
  General1DFunction Stellar_Age_Lum ;
  // optical depth in the V band from side to side
  General1DFunction Tau_V ;


 
  bool has_sfr, has_stellar_mass, has_stellar_wd_mass, has_gas_metallicity, has_star_lum_metallicity, has_stellar_age_mass,has_stellar_age_lum, has_tauv  ;
  bool Has_Star_Lum_Metallicity()const {return(has_star_lum_metallicity);}
  bool Has_Gas_Metallicity()const {return(has_gas_metallicity);}
  bool Has_SFR()const {return(has_sfr);}
  bool Has_Stellar_Mass() const {return(has_stellar_mass);}
  bool Has_Stellar_WD_Mass() const {return(has_stellar_wd_mass);}
  bool Has_Stellar_Age_Mass() const {return(has_stellar_age_mass);}
  bool Has_Stellar_Age_Lum() const {return(has_stellar_age_lum);}
  bool Has_Tau_V() const {return(has_tauv);}

  // sfr 
  double S_F_R(double age) const ;
  // log10(ssfr), integrated between age-500 Myr and age. ssfr= integsfr*1e-6/stell_mass
  double sSFR(double age) const ;
  // normalised mass in stars (for Mtot = 1 solar mass)
  double StellarMass(double age) const ;
  // normalised mass in white dwarfs (for Mtot = 1 solar mass)
  double StellarWDMass(double age) const ;
  // mean stellar age averaged on mass Myr
  double Mean_Stellar_Age_Mass(double age) const ;
  // mean stellar age averaged on bolometric lum. Myr
  double Mean_Stellar_Age_Lum(double age) const ;
  //
  double GasMetallicity(double age) const ;
  double StarLumMetallicity(double age) const ;
  double TauV(double age) const ;

  GalaxyTemplate(const std::string& cardfilename="");


  //beware (y,x) !!
  double Value(const double& wavelength, const double& type_index) const;
  Grid2DXSlice ContinuumSpectrum(const double& Type) const;
  General1DFunction LineSpectrum(const double& Type) const;
  double MinType() const {return continuum.Xmin();};
  double MaxType() const {return continuum.Xmax();};
  double MinWavelength() const {return continuum.Ymin();};
  double MaxWavelength() const {return continuum.Ymax();};
  
};

#endif // GALAXY_TEMPLATE_H
