// -*- C++ -*-
// 
// \file globalval.h
// 
// Last modified: $Date: 2014/04/29 13:29:32 $
// By:            $Author: nrl $
// 

#ifndef GLOBALVAL__H
#define GLOBALVAL__H

#include <string>
#include <list>
#include <vector>
#include <map>
#include <iostream>


//! to store in files things like "Key value(s)" things.
class GlobalVal : private  std::map<std::string, std::vector<std::string> > {

private :
  
  //! default=false , true when a key @ALLOW_MULTIPLE_TAGS is read
  bool allow_multiple_tags;

public :

  GlobalVal() : allow_multiple_tags(false) {};

  bool AddKey(const std::string &Key, const std::vector<std::string> &Values);
  
  bool AddKey(const std::string &Key, const std::string &Value);
  
  bool AddKey(const std::string& Key, const std::list<std::string>& Values);
  
  bool AddKey(const std::string &Key, const std::vector<double> &Values);
  
  bool AddKey(const std::string &Key, const double &Value);

  bool AddKey(const std::string& Key, const std::list<double>& Values);
  
  unsigned long NKey() const;

  bool HasKey(const std::string &Key) const;
  
  std::string         getStringValue(const std::string& Key, bool fatal_if_failed = true) const;
  
  std::vector<std::string> getStringValues(const std::string& Key, bool fatal_if_failed = true) const;
  
  double         getDoubleValue(const std::string& Key, bool fatal_if_failed = true) const;
  
  std::vector<double> getDoubleValues(const std::string& Key, bool fatal_if_failed = true) const;

  std::vector<std::string> OutputLines() const;

  std::vector<std::string> Keys() const;

  void Erase(const std::string& Key);
  
  bool ProcessLine(char *Line);
  
  void Read(const std::string &FileName, bool break_when_digit=true);
  // this constructor is to be used when you only need to read '@' lines in a file
  GlobalVal(const std::string &FileName, bool break_when_digit=true);

  //  GlobalVal(const GlobalVal &O) { cout << " on copie un globalval " << endl;}


private:
  template<typename T>
  bool GenericAddKey(const std::string& Key, const T& Values);
};

std::ostream& operator <<(std::ostream &S, const GlobalVal &G);

#endif /* GLOBALVAL__H */
