#include <iostream>
#include <fstream>
#include <string>


#include "intmatvect.h"
 


IntMat::IntMat(const size_t NX, const size_t NY) { 
 data=NULL;
 nx=ny=0;
 allocate(NX,NY);
}

IntMat::IntMat(const IntMat& other) {
  data=NULL;
  nx=ny=0;
  allocate(other.nx,other.ny);
  memcpy(data,other.data,sizeof(int)*nx*ny); 
}

void IntMat::allocate(const size_t NX, const size_t NY) {
  if(nx!=NX || ny!=NY) {
    nx=NX; 
    ny=NY; 
    if (data) 
      delete [] data;
    data = new int[nx*ny];
  } 
  Zero();
}

int IntMat::writeASCII(const std::string &FileName) const
{
  std::ofstream S(FileName.c_str());
  if (!S)
    {
      std::cout << " IntMat::writeASCII() : cannot open " << FileName << std::endl;
      return 0;
    }
  int status = writeASCII(S);
  S.close();
  return status;
}

int IntMat::writeASCII(std::ostream& Stream) const {
  Stream << nx << " " << ny << std::endl;
  
  for(size_t j=0;j<ny;j++) {
    //Stream << "0.." << nx-1 << "," << j << ": ";
    for(size_t i=0;i<nx;i++) {
      Stream << " " << (*this)(i,j);
    }
    Stream << std::endl;
  }
  return 0;
}


int IntMat::readASCII(const std::string &FileName)
{
  std::ifstream S(FileName.c_str());
  if (!S)
    {
      std::cout << " IntMat::readASCII() : cannot open " << FileName << std::endl;
      return -1;
    }
  int status = readASCII(S);
  S.close();
  return status;
}


int IntMat::readASCII(std::istream& Stream) {
  size_t  fnx,fny;
  
  Stream >> fnx >> fny;
  allocate(fnx,fny);
  
  int val;
  for(size_t j=0;j<ny;j++) {
    for(size_t i=0;i<nx;i++) {
      if(Stream >> val)
	(*this)(i,j)=val;
      else{
	std::cerr << "error reading MATRIX" << std::endl;
	abort();
      }
    }
  }
  return 1;
}

IntMat IntMat::SubBlock
(size_t x_min,size_t x_max,size_t y_min,size_t y_max) const {
   if( x_max >= nx || y_max>= ny ) {
    std::cout << "IntMat::SubBlockFromIndexes ERROR, trying to get a sub-matrix with indices" << std::endl;
    std::cout << "x_min,x_max,y_min,y_max = "
	 << x_min << ","
	 << x_max << ","
    	 << y_min << ","
	 << y_max << std::endl;
    std::cout << "nx,ny = "<< nx << "," << ny << std::endl;
    abort();
  }
  size_t nx_new = (x_max-x_min+1);
  size_t ny_new = (y_max-y_min+1);
  IntMat res(nx_new,ny_new);
  for(size_t j=0;j<ny_new;++j)
    for(size_t i=0;i<nx_new;++i)
      res(i,j) = (*this)(i+x_min,j+y_min);
  return res;

}
