#ifndef INTMATVECT__H
#define INTMATVECT__H

#define INTMAT_CHECK_BOUNDS

#include <cstdlib>
#include <cstring>

class IntMat {
  protected:
  int *data;
  size_t nx,ny;
  
  
 public:
  
  IntMat() : data(NULL), nx(0), ny(0) {};  
  IntMat(const size_t NX, const size_t NY);
  IntMat(const IntMat& other);
  ~IntMat() { if(data) delete [] data;}
  
  void allocate(const size_t NX, const size_t NY);
  
  inline int& operator () (const size_t i, const size_t j) const {
#ifdef INTMAT_CHECK_BOUNDS
    if (i>=nx || j>=ny) { 
      std::cout << "IntMat::operator () overflow i,j nx,ny " 
	 << i << "," << j << " "
	   << nx << "," << ny << " "
	   << std::endl;
      abort();
    }
#endif
    return data[i+j*nx];
  }
  
  
  size_t SizeX() const { return nx;}
  size_t SizeY() const { return ny;}
  
  void Zero() {memset(data, 0, nx*ny*sizeof(int));};
  void Identity();

  const int* Data() const {return data;};
  int* NonConstData() {return data;};

  
  friend std::ostream& operator << (std::ostream &stream, const IntMat &m)
    { m.writeASCII(stream); return stream;}
  
  int readASCII(std::istream& Stream);
  int readASCII(const std::string &FileName);
  int writeASCII(std::ostream& Stream) const;
  int writeASCII(const std::string &FileName) const;
  
  // IntMat transposed() const;

    // get a block of this matrix as a new matrix
  // size (x_max-x_min+1)*(y_max-y_min+1) (both min and max included)
  // remember  0 <= x < nx ,  0 <= y < ny
  IntMat SubBlock
    (size_t x_min,size_t x_max,size_t y_min,size_t y_max) const;
  
  IntMat & operator =(const IntMat& Right) {
    allocate(Right.nx,Right.ny);
    memcpy(data,Right.data,nx*ny*sizeof(int));
    return (*this);
  }
};





#endif 
