#ifndef SALTNIRMODEL__H
#define SALTNIRMODEL__H

#include <model2fit.h>
#include "salt2extinction.h"
#include "sampledfunction.h"

class SimpleExtinction 
{
 private:
  General1DFunction extinction;
 public :
  SimpleExtinction() {};
  SimpleExtinction(const std::string &FileName);
  void Read(const std::string &FileName);
  double extinction_lambda(const double &Color, const double &Lambda) const;
};



class SaltNirModel : public Model2Fit {
  LcParam *pDayMax;
  LcParam *pRedshift;
  LcParam *pColor; 
  LcParam *pX1;
  LcParam *pX0;
  Grid2DFunction sed;
  Grid2DFunction meanCorr,dm15Corr;
  double phaseMin,phaseMax; // useless
  SimpleExtinction extinction_law;

 public :

  SaltNirModel();

  std::string ModelName() const { return "SaltNirModel"; }

  double Flux(const Filter *F, const double observerday) const;

  //! init parameters before starting a minuit fit
  void InitParams(const MeasuredData* data);

  bool CanComputeFluxFor(const Filter& observerfilter, bool quiet=false) const;

  double GetPhase(double date) const;

  const  Grid2DFunction& MeanCorr() const { return meanCorr;}
  const  Grid2DFunction& Dm15Corr() const { return dm15Corr;}


};

#endif /* SALTNIRMODEL__H */
